@@ -19,18 +19,10 @@ PLATFORM_LINUX
 #undef ODS
 #undef CATCH_ERRORS
 #undef ASSERT
-#undef _totlower
 #undef ZeroMemory
 #undef __forceinline
 #endif
 
-/**************************************************************************************************
-64-bit
-**************************************************************************************************/
-#if defined(_WIN64) || defined(__x86_64__)
-#define APE_64_BIT
-#endif
-
 /**************************************************************************************************
 Warnings
 **************************************************************************************************/
@@ -46,12 +38,12 @@ Override (define in MSVC)
 #endif
 
 /**************************************************************************************************
-NULL (define in MSVC)
+NULL (define to nullptr on all platforms)
 **************************************************************************************************/
-#ifdef _MSC_VER
+#if __cplusplus >= 201103L
 #define APE_NULL nullptr
 #else
-#define APE_NULL NULL
+#define APE_NULL 0
 #endif
 
 /**************************************************************************************************
@@ -111,13 +103,14 @@ Version
 #include "Version.h"
 
 // year in the copyright strings
-#define MAC_YEAR 2022
+#define MAC_YEAR 2023
 
 // build the version string
 #define STRINGIZE2(s) #s
 #define STRINGIZE(s) STRINGIZE2(s)
 #define MAC_VER_FILE_VERSION_STR                        STRINGIZE(MAC_VERSION_MAJOR) _T(".") STRINGIZE(MAC_VERSION_REVISION)
 #define MAC_VER_FILE_VERSION_STR_NARROW                 STRINGIZE(MAC_VERSION_MAJOR) "." STRINGIZE(MAC_VERSION_REVISION)
+#define MAC_VER_FILE_VERSION_STR_WIDE                   STRINGIZE(MAC_VERSION_MAJOR) L"." STRINGIZE(MAC_VERSION_REVISION)
 
 #define MAC_FILE_VERSION_NUMBER                         3990
 #define MAC_VERSION_STRING                              MAC_VER_FILE_VERSION_STR
@@ -128,28 +121,12 @@ Version
 #define MAC_RESOURCE_VERSION_COMMA                      MAC_VERSION_MAJOR, MAC_VERSION_REVISION, 0, 0
 #define MAC_RESOURCE_VERSION_STRING                     MAC_VER_FILE_VERSION_STR
 #define MAC_RESOURCE_COPYRIGHT                          "Copyright (c) 2000-" STRINGIZE(MAC_YEAR) " Matthew T. Ashland"
-#define CONSOLE_NAME                                    _T("--- Monkey's Audio Console Front End (v ") MAC_VER_FILE_VERSION_STR _T(") (c) Matthew T. Ashland ---\n")
+#define CONSOLE_NAME                                    L"--- Monkey's Audio Console Front End (v " MAC_VER_FILE_VERSION_STR_WIDE L") (c) Matthew T. Ashland ---\n"
 #define PLUGIN_ABOUT                                    _T("Monkey's Audio Player v") MAC_VER_FILE_VERSION_STR _T("\nCopyrighted (c) 2000-") STRINGIZE(MAC_YEAR) _T(" by Matthew T. Ashland")
 
 /**************************************************************************************************
 Global compiler settings (useful for porting)
 **************************************************************************************************/
-// assembly code (helps performance, but limits portability)
-#if defined __SSE2__
-    #define ENABLE_SSE_ASSEMBLY
-#endif
-
-#if defined __AVX2__
-    #define ENABLE_AVX_ASSEMBLY
-#endif
-
-#ifdef _MSC_VER // doesn't compile in gcc
-    #if defined(_M_IX86) || defined(_M_X64)
-        #define ENABLE_SSE_ASSEMBLY
-        #define ENABLE_AVX_ASSEMBLY
-    #endif
-#endif
-
 // APE_BACKWARDS_COMPATIBILITY is only needed for decoding APE 3.92 or earlier files.  It
 // has not been possible to make these files for over 10 years, so it's unlikely
 // that disabling APE_BACKWARDS_COMPATIBILITY would have any effect on a normal user.  For
@@ -162,6 +139,9 @@ Global compiler settings (useful for porting)
 // disable this to turn off compression code
 #define APE_SUPPORT_COMPRESS
 
+// flip this to enable float compression
+#define APE_SUPPORT_FLOAT_COMPRESSION
+
 // compression modes
 #define ENABLE_COMPRESSION_MODE_FAST
 #define ENABLE_COMPRESSION_MODE_NORMAL
@@ -190,7 +170,6 @@ namespace APE
     // string types
     typedef char                                        str_ansi;
     typedef unsigned char                               str_utf8;
-    typedef int16                                       str_utf16;
     typedef wchar_t                                     str_utfn; // could be UTF-16 or UTF-32 depending on platform
 }
 
@@ -203,8 +182,6 @@ Global macros
 #define WAVE_FORMAT_ALAW 0x0006
 #define WAVE_FORMAT_MULAW 0x0007
 
-#define APE_TRUNCATE ((size_t)-1)
-
 // we use this to check for zero because Clang warns if we just equate a double with zero
 #define APE_DOUBLE_ZERO 0.00001
 
@@ -215,10 +192,13 @@ Global macros
     #define DLLEXPORT                                   __declspec(dllexport)
     #define SLEEP(MILLISECONDS)                         ::Sleep(MILLISECONDS)
     #define MESSAGEBOX(PARENT, TEXT, CAPTION, TYPE)     ::MessageBox(PARENT, TEXT, CAPTION, TYPE)
-    #define PUMP_MESSAGE_LOOP                           { MSG Msg; while (PeekMessage(&Msg, NULL, 0, 0, PM_REMOVE) != 0) { TranslateMessage(&Msg); DispatchMessage(&Msg); } }
     #define ODS                                         OutputDebugString
     #define TICK_COUNT_TYPE                             unsigned long long
-    #define TICK_COUNT_READ(VARIABLE)                   VARIABLE = GetTickCount64()
+    #if _WIN32_WINNT >= 0x600
+        #define TICK_COUNT_READ(VARIABLE)               VARIABLE = GetTickCount64()
+    #else
+        #define TICK_COUNT_READ(VARIABLE)               VARIABLE = GetTickCount()
+    #endif
     #define TICK_COUNT_FREQ                             1000
 
     #if !defined(ASSERT)
@@ -228,12 +208,26 @@ Global macros
             #define ASSERT(e)
         #endif
     #endif
+
+    #if _WIN32_WINNT < 0x600
+        #define wcsncpy_s(A, B, C, D) wcsncpy(A, C, D)
+        #define wcscpy_s(A, B, C) wcscpy(A, C)
+        #define wcscat_s(A, B, C) wcscat(A, C)
+        #define strcpy_s(A, B, C) strcpy(A, C)
+
+        #undef _tcsncpy_s
+        #undef _tcscpy_s
+        #undef _stprintf_s
+
+        #define _tcsncpy_s(A, B, C, D) _tcsncpy(A, C, D)
+        #define _tcscpy_s(A, B, C) _tcscpy(A, C)
+        #define _stprintf_s(A, B, C, ...) _stprintf(A, C, __VA_ARGS__)
+    #endif
 #else
     #define IO_USE_STD_LIB_FILE_IO
-    #define DLLEXPORT
+    #define DLLEXPORT                                   __attribute__ ((visibility ("default")))
     #define SLEEP(MILLISECONDS)                         { struct timespec t; t.tv_sec = (MILLISECONDS) / 1000; t.tv_nsec = (MILLISECONDS) % 1000 * 1000000; nanosleep(&t, NULL); }
     #define MESSAGEBOX(PARENT, TEXT, CAPTION, TYPE)
-    #define PUMP_MESSAGE_LOOP
     #undef  ODS
     #define ODS                                         printf
     #define TICK_COUNT_TYPE                             unsigned long long
@@ -241,16 +235,37 @@ Global macros
     #define TICK_COUNT_FREQ                             1000000
     #undef  ASSERT
     #define ASSERT(e)
-    #define wcsncpy_s(A, B, C, D) wcsncpy(A, C, D)
-    inline wchar_t * wcscpy_s(wchar_t * dest, size_t destsz, const wchar_t * src)
-    {
-        (void) destsz; // this avoids an unreferenced variable
-        return wcscpy(dest, src);
-    }
-    #define wcscat_s(A, B, C) wcscat(A, C)
-    #define sprintf_s(A, B, C, D) sprintf(A, C, D)
-    #define strcpy_s(A, B, C) strcpy(A, C)
-    #define _tcscat_s(A, B, C) _tcscat(A, C)
+
+    #ifndef __STDC_LIB_EXT1__
+        inline int wcsncpy_s(wchar_t * dest, size_t destsz, const wchar_t * src, size_t count)
+        {
+            if (!dest || !src || destsz == 0 || count == 0 || (count >= destsz && destsz <= wcslen(src)))
+                return -1;
+            wcsncpy(dest, src, count);
+            return 0;
+        }
+        inline int wcscpy_s(wchar_t * dest, size_t destsz, const wchar_t * src)
+        {
+            if (!dest || !src || destsz == 0 || destsz <= wcslen(src))
+                return -1;
+            wcscpy(dest, src);
+            return 0;
+        }
+        inline int wcscat_s(wchar_t * dest, size_t destsz, const wchar_t * src)
+        {
+            if (!dest || !src || destsz == 0 || destsz <= wcslen(dest) + wcslen(src))
+                return -1;
+            wcscat(dest, src);
+            return 0;
+        }
+        inline int strcpy_s(char * dest, size_t destsz, const char * src)
+        {
+            if (!dest || !src || destsz == 0 || destsz <= strlen(src))
+                return -1;
+            strcpy(dest, src);
+            return 0;
+        }
+    #endif
 #endif
 
 /**************************************************************************************************
@@ -303,6 +318,8 @@ Global defines
 #define APE_BYTES_IN_MEGABYTE            1048576
 #define APE_BYTES_IN_GIGABYTE            APE::int64(1073741824)
 
+#define APE_WAV_HEADER_OR_FOOTER_MAXIMUM_BYTES (APE_BYTES_IN_MEGABYTE * 8)
+
 /**************************************************************************************************
 Byte order
 **************************************************************************************************/
@@ -319,23 +336,22 @@ Channels
 /**************************************************************************************************
 Macros
 **************************************************************************************************/
-#define MB(TEST) MESSAGEBOX(NULL, TEST, _T("Information"), MB_OK);
-#define MBN(NUMBER) { TCHAR cNumber[16]; _stprintf(cNumber, _T("%d"), NUMBER); MESSAGEBOX(NULL, cNumber, _T("Information"), MB_OK); }
+#define MB(TEST) MESSAGEBOX(APE_NULL, TEST, _T("Information"), MB_OK);
+#define MBN(NUMBER) { TCHAR cNumber[16]; _stprintf(cNumber, _T("%d"), NUMBER); MESSAGEBOX(APE_NULL, cNumber, _T("Information"), MB_OK); }
 
-#define SAFE_DELETE(POINTER) if (POINTER) { delete POINTER; POINTER = NULL; }
-#define SAFE_ARRAY_DELETE(POINTER) if (POINTER) { delete [] POINTER; POINTER = NULL; }
-#define SAFE_VOID_CLASS_DELETE(POINTER, Class) { Class *pClass = (Class *) POINTER; if (pClass) { delete pClass; POINTER = NULL; } }
-#define SAFE_FILE_CLOSE(HANDLE) if (HANDLE != INVALID_HANDLE_VALUE) { CloseHandle(HANDLE); HANDLE = INVALID_HANDLE_VALUE; }
+#define APE_SAFE_DELETE(POINTER) if (POINTER) { delete POINTER; POINTER = APE_NULL; }
+#define APE_SAFE_ARRAY_DELETE(POINTER) if (POINTER) { delete [] POINTER; POINTER = APE_NULL; }
+#define APE_SAFE_FILE_CLOSE(HANDLE) if (HANDLE != INVALID_HANDLE_VALUE) { CloseHandle(HANDLE); HANDLE = INVALID_HANDLE_VALUE; }
 
 #define ODN(NUMBER) { TCHAR cNumber[16]; _stprintf(cNumber, _T("%d\n"), static_cast<int>(NUMBER)); ODS(cNumber); }
 
 #define CATCH_ERRORS(CODE) try { CODE } catch(...) { }
 
-#define RETURN_ON_ERROR(FUNCTION) {    int nFunctionResult = FUNCTION; if (nFunctionResult != 0) { return nFunctionResult; } }
+#define RETURN_ON_ERROR(FUNCTION) { const int nFunctionResult = static_cast<int>(FUNCTION); if (nFunctionResult != ERROR_SUCCESS) { return nFunctionResult; } }
 #define RETURN_VALUE_ON_ERROR(FUNCTION, VALUE) { int nFunctionResult = FUNCTION; if (nFunctionResult != 0) { return VALUE; } }
 #define RETURN_ON_EXCEPTION(CODE, VALUE) { try { CODE } catch(...) { return VALUE; } }
 
-#define THROW_ON_ERROR(CODE) { intn nThrowResult = static_cast<intn> (CODE); if (nThrowResult != 0) throw(nThrowResult); }
+#define THROW_ON_ERROR(CODE) { const intn nThrowResult = static_cast<intn> (CODE); if (nThrowResult != 0) throw(nThrowResult); }
 
 #define EXPAND_8_TIMES(CODE) CODE CODE CODE CODE CODE CODE CODE CODE
 #define EXPAND_9_TIMES(CODE) CODE CODE CODE CODE CODE CODE CODE CODE CODE
